package org.zte.gm.tool;

package ..deTemp.1731157327760;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.stream.Collectors;

public class SqlExecutor {
  private static final Logger LOGGER = Logger.getLogger(org.zte.gm.tool.SqlExecutor.class.getName());
  
  static {
    LOGGER.setLevel(Level.ALL);
  }
  
  private static String JDBC_URL = "jdbc:sqlite:";
  
  private static String JDBC_USER = "";
  
  private static String JDBC_PASSWORD = "";
  
  private static String SQL_DIRECTORY = "";
  
  public static void main(String[] args) {
    if (args.length != 1) {
      LOGGER.log(Level.WARNING, "请提供要读取SQL文件的目录路径作为参数。");
      return;
    } 
    String dir = args[0];
    JDBC_URL = "jdbc:sqlite:" + dir + "gmwebserver.sqlite";
    SQL_DIRECTORY = dir;
    executeSQLFilesInDirectory(SQL_DIRECTORY);
  }
  
  public static void executeSQLFilesInDirectory(String directoryPath) {
    File directory = new File(directoryPath);
    if (!directory.exists() || !directory.isDirectory()) {
      LOGGER.log(Level.SEVERE, "指定的目录不存在或不是一个有效的目录。");
      return;
    } 
    File[] sqlFiles = directory.listFiles((dir, name) -> name.toLowerCase().endsWith(".sql"));
    if (sqlFiles == null) {
      LOGGER.log(Level.WARNING, "目录中没有找到SQL文件。");
      return;
    } 
    try(Connection connection = DriverManager.getConnection(JDBC_URL, JDBC_USER, JDBC_PASSWORD); 
        Statement statement = connection.createStatement()) {
      ResultSet resultSet1 = statement.executeQuery("SELECT count(name) FROM sqlite_master WHERE type='table' AND name='update_version';");
      int anInt = resultSet1.getInt(1);
      if (anInt == 0) {
        statement.execute("create table update_version(version integer);");
        statement.execute("insert into update_version(version) values(0)");
      } 
      ResultSet resultSet = statement.executeQuery("select version from update_version;");
      int version = resultSet.getInt("version");
      LOGGER.log(Level.INFO, "当前数据库版本为：" + version);
      List<File> needExecuteFiles = (List<File>)Arrays.<File>stream(sqlFiles).filter(file -> {
            try {
              int i = Integer.parseInt(file.getName().replace(".sql", ""));
              return (i > version);
            } catch (NumberFormatException e) {
              LOGGER.log(Level.INFO, "非法sql文件：" + file.getName());
              return false;
            } 
          }).sorted(Comparator.comparingInt(file -> Integer.parseInt(file.getName().replace(".sql", "")))).collect(Collectors.toList());
      LOGGER.log(Level.INFO, "开始执行SQL文件。。。。");
      for (File sqlFile : needExecuteFiles)
        executeSQLFile(statement, sqlFile); 
    } catch (SQLException e) {
      LOGGER.log(Level.SEVERE, "数据库连接或执行过程中出现错误", e);
    } 
  }
  
  private static void executeSQLFile(Statement statement, File sqlFile) {
    try (BufferedReader reader = new BufferedReader(new FileReader(sqlFile))) {
      StringBuilder sqlStatement = new StringBuilder();
      boolean inMultiLineComment = false;
      String line;
      while ((line = reader.readLine()) != null) {
        line = line.trim();
        if (line.isEmpty())
          continue; 
        if (inMultiLineComment)
          if (line.contains("*/")) {
            inMultiLineComment = false;
            line = line.substring(line.indexOf("*/") + 2).trim();
          } else {
            continue;
          }  
        if (line.startsWith("--"))
          continue; 
        if (line.startsWith("/*"))
          if (line.contains("*/")) {
            line = line.substring(line.indexOf("*/") + 2).trim();
          } else {
            inMultiLineComment = true;
            continue;
          }  
        sqlStatement.append(line);
        if (line.endsWith(";")) {
          statement.execute(sqlStatement.toString());
          LOGGER.log(Level.INFO, "成功执行 " + sqlFile.getName() + " 中的SQL语句：" + sqlStatement);
          sqlStatement.setLength(0);
        } 
      } 
      LOGGER.log(Level.INFO, "成功执行 " + sqlFile.getName() + " 中的所有SQL语句。");
      int replace = Integer.parseInt(sqlFile.getName().replace(".sql", ""));
      int i = statement.executeUpdate("update update_version set version = " + replace);
      if (i == 1) {
        LOGGER.log(Level.INFO, "更新数据库版本为：" + replace);
      } else {
        LOGGER.log(Level.INFO, "更新数据库版本失败");
      } 
    } catch (Exception e) {
      LOGGER.log(Level.SEVERE, "执行 " + sqlFile.getName() + " 中的SQL语句时出错", e);
    } 
  }
}
